/**
 * FileMd5Util.java
 * com.bibenet.cedp.utils
 * <p>
 * ****************************************************************************
 * Change Log
 * <p>
 * 1. wangyu create me at 2016年7月16日上午9:48:14 for class DESC.
 * 2.
 * ****************************************************************************
 * Copyright (c) 2016, www.bibenet.com All Rights Reserved.O(∩_∩)O
 */

package com.flyfish.framework.file.utils;


import com.flyfish.framework.utils.Assert;

import java.io.*;
import java.math.BigInteger;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;


/**
 * @author wangyu
 * @name <p>FileMD5Util</p>
 * @desc 文件MD5校验工具 <1GB
 */
public class FileMD5Utils {

    /**
     * <p>getMd5ByFile</p>
     * 获取文件的Md5
     *
     * @param file 文件
     * @return md5
     */
    public static String getMd5ByFile(File file) {

        FileInputStream in = null;
        try {
            in = new FileInputStream(file);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        return getMd5ByInputStream(in, file.length());
    }

    /**
     * <p>getMd5ByInputStream</p>
     * 获取流的Md5
     *
     * @param in     输入流
     * @param length 文件长度
     * @return md5
     */
    public static String getMd5ByInputStream(InputStream in, Long length) {
        String value = null;
        Assert.notNull(in, "计算md5的输入流为null！");
        if (in instanceof FileInputStream) {
            try {
                MappedByteBuffer byteBuffer = ((FileInputStream) in).getChannel().map(FileChannel.MapMode.READ_ONLY, 0, length);
                MessageDigest md5 = MessageDigest.getInstance("MD5");
                md5.update(byteBuffer);
                BigInteger bi = new BigInteger(1, md5.digest());
                value = bi.toString(16);
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                try {
                    in.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        } else {
            try {
                value = getMd5ByASCIIStream(in, length);
            } catch (NoSuchAlgorithmException | IOException e) {
                e.printStackTrace();
            } finally {
                try {
                    in.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return value;
    }

    private static String getMd5ByASCIIStream(InputStream in, Long length) throws NoSuchAlgorithmException, IOException {
        StringBuilder md5 = new StringBuilder();
        MessageDigest md = MessageDigest.getInstance("MD5");
        byte[] dataBytes = new byte[1024];

        int nread;
        while ((nread = in.read(dataBytes)) != -1) {
            md.update(dataBytes, 0, nread);
        }
        byte[] mdbytes = md.digest();

        // 转换二进制到HEX
        for (byte mdByte : mdbytes) {
            md5.append(Integer.toString((mdByte & 0xff) + 0x100, 16).substring(1));
        }
        return md5.toString();
    }

    public static String getMd5ByPath(String path, Long length) {
        Assert.notNull(path, "md5验证时文件路径为空！");
        try {
            return getMd5ByInputStream(new FileInputStream(path), length);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        return null;
    }
}

